@@ -438,7 +438,8 @@ bool cmdline_applyaction(cmdline_pkgaction_type action,
 	if (verbose > 0)
 	  printf(_("Setting package %s on hold\n"), pkg.FullName(true).c_str());
 
-	(*apt_cache_file)->mark_keep(pkg, false, true, NULL);
+	fprintf(stderr, "This code path should not be reached (now implemented in cmdline_mark)\n");
+	(*apt_cache_file)->mark_keep(pkg, is_auto_installed(pkg), true, NULL);
       }
       break;
     case cmdline_keep:
@@ -446,7 +447,8 @@ bool cmdline_applyaction(cmdline_pkgaction_type action,
 	if (verbose > 0)
 	  printf(_("Marking package %s as keep\n"), pkg.FullName(true).c_str());
 
-	(*apt_cache_file)->mark_keep(pkg, false, false, NULL);
+	fprintf(stderr, "This code path should not be reached (now implemented in cmdline_mark)\n");
+	(*apt_cache_file)->mark_keep(pkg, is_auto_installed(pkg), false, NULL);
       }
       break;
     case cmdline_unhold:
@@ -455,7 +457,8 @@ bool cmdline_applyaction(cmdline_pkgaction_type action,
 	  if (verbose > 0)
 	    printf(_("Setting package %s as not on hold\n"), pkg.FullName(true).c_str());
 
-	  (*apt_cache_file)->mark_keep(pkg, false, false, NULL);
+	  fprintf(stderr, "This code path should not be reached (now implemented in cmdline_mark)\n");
+	  (*apt_cache_file)->mark_keep(pkg, is_auto_installed(pkg), false, NULL);
 	}
       else
 	{
@@ -467,6 +470,7 @@ bool cmdline_applyaction(cmdline_pkgaction_type action,
 	if (verbose > 0)
 	  printf(_("Marking package %s as automatically installed\n"), pkg.FullName(true).c_str());
 
+	fprintf(stderr, "This code path should not be reached (now implemented in cmdline_mark)\n");
 	(*apt_cache_file)->mark_auto_installed(pkg, true, NULL);
       }
       break;
@@ -475,6 +479,7 @@ bool cmdline_applyaction(cmdline_pkgaction_type action,
 	if (verbose > 0)
 	  printf(_("Unmarking package %s as automatically installed\n"), pkg.FullName(true).c_str());
 
+	fprintf(stderr, "This code path should not be reached (now implemented in cmdline_mark)\n");
 	(*apt_cache_file)->mark_auto_installed(pkg, false, NULL);
       }
       break;
