@@ -1,6 +1,7 @@
 // cmdline_extract_cache_subset.cc
 //
 //   Copyright (C) 2008-2010 Daniel Burrows
+//   Copyright (C) 2015-2016 Manuel A. Fernandez Montecelo
 //
 //   This program is free software; you can redistribute it and/or
 //   modify it under the terms of the GNU General Public License as
@@ -14,8 +15,8 @@
 //
 //   You should have received a copy of the GNU General Public License
 //   along with this program; see the file COPYING.  If not, write to
-//   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-//   Boston, MA 02111-1307, USA.
+//   the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+//   Boston, MA 02110-1301, USA.
 
 
 // Local includes:
@@ -65,12 +66,10 @@ namespace aptitude
 
       std::shared_ptr<OpProgress> progress = make_text_progress(false, term, term, term);
 
-      apt_init(progress.get(), true);
-      if(_error->PendingError())
-	{
-	  _error->DumpErrors();
-	  return -1;
-	}
+      bool operation_needs_lock = true;
+      apt_init(progress.get(), true, operation_needs_lock, nullptr);
+
+      aptitude::cmdline::on_apt_errors_print_and_die();
 
       bool ok = true;
       std::set<pkgCache::PkgIterator> packages;
@@ -139,7 +138,7 @@ namespace aptitude
 
       bool copy_ok = !_error->PendingError();
 
-      _error->DumpErrors();
+      aptitude::cmdline::on_apt_errors_print_and_die();
 
       return copy_ok ? 0 : 1;
     }
