@@ -14,8 +14,8 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+// Boston, MA 02110-1301, USA.
 
 // Local includes:
 #include "acquire_download_progress.h"
@@ -152,9 +152,7 @@ namespace aptitude
                              it->CurrentItem->Owner->Complete,
                              it->CurrentItem->ShortDesc,
                              if_not_zero(it->CurrentItem->Owner->ID),
-                             it->CurrentItem->Owner->Mode == NULL
-                               ? ""
-                               : it->CurrentItem->Owner->Mode);
+                             it->CurrentItem->Owner->ActiveSubprocess);
 
               active_downloads.push_back(current_file);
             }
