@@ -1,7 +1,7 @@
 // pkg_grouppolicy.cc
 //
 //  Copyright 1999-2005, 2007-2010 Daniel Burrows
-//  Copyright 2012-2015 Manuel A. Fernandez Montecelo
+//  Copyright 2012-2016 Manuel A. Fernandez Montecelo
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -15,8 +15,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; see the file COPYING.  If not, write to
-//  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-//  Boston, MA 02111-1307, USA.
+//  the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+//  Boston, MA 02110-1301, USA.
 //
 //  Implementation of stuff that needs implementing.
 
@@ -58,34 +58,6 @@ pkg_grouppolicy_factory::~pkg_grouppolicy_factory()
 {
 }
 
-// This special tree munges its tag to allow an integer to be prepended to it.
-// Ok, it's a dreadful hack.  I admit it.
-class pkg_subtree_with_order:public pkg_subtree
-{
-  wstring my_tag;
-public:
-  pkg_subtree_with_order(wstring name, wstring description,
-			 sigc::signal1<void, std::wstring> *_info_signal,
-			 unsigned char order, bool expand=false)
-    :pkg_subtree(name, description, _info_signal, expand)
-  {
-    my_tag+=order;
-    my_tag+=pkg_subtree::tag();
-  }
-
-  pkg_subtree_with_order(wstring name, unsigned char order, bool expand=false)
-    :pkg_subtree(name, expand)
-  {
-    my_tag+=order;
-    my_tag+=pkg_subtree::tag();
-  }
-
-  virtual const wchar_t *tag()
-  {
-    return my_tag.c_str();
-  }
-};
-
 // The following class is a special policy which is used to terminate a
 // policy chain:
 class pkg_grouppolicy_end:public pkg_grouppolicy
@@ -125,6 +97,9 @@ class pkg_grouppolicy_section:public pkg_grouppolicy
 
   // The descriptions are in the cw::style used by package descriptions.
   static std::map<string, wstring> section_descriptions;
+  // Width for section names -- will be calculated to fill name width and indent
+  // descriptions as if they were columns -- #103416
+  static size_t section_name_width;
   // The available top sections (i.e. main, contrib, non-free) in order.
   static std::vector<string> top_sections;
   static void init_section_data();
@@ -159,6 +134,7 @@ pkg_grouppolicy *pkg_grouppolicy_section_factory::instantiate(pkg_signal *_sig,
 
 std::map<string, wstring> pkg_grouppolicy_section::section_descriptions;
 std::vector<string> pkg_grouppolicy_section::top_sections;
+size_t pkg_grouppolicy_section::section_name_width = 0;
 
 namespace
 {
@@ -244,6 +220,15 @@ void pkg_grouppolicy_section::init_section_data()
 	}
     }
 
+  // width of section names -- for indenting descriptions
+  for (const auto& it : section_descriptions)
+    {
+      if (it.first.size() > section_name_width)
+	section_name_width = it.first.size();
+    }
+  // extra indentation
+  section_name_width += 2;
+
   // Read in the list of Top-Sections; do not use a cached value.
   top_sections = aptitude::apt::get_top_sections(false);
 
@@ -381,12 +366,22 @@ void pkg_grouppolicy_section::add_package(const pkgCache::PkgIterator &pkg,
 	      wstring desc;
 	      if(section_descriptions.find(section_tail) != section_descriptions.end())
 		{
+		  size_t current_section_name_length = cw::util::transcode(section).length();
+		  size_t current_section_name_fill = 0;
+		  if (section_name_width > current_section_name_length)
+		    current_section_name_fill = section_name_width - current_section_name_length;
+		  else
+		    current_section_name_fill = 1;
+		  wstring fill(current_section_name_fill, L' ');
+
 		  desc = section_descriptions[section_tail];
 		  if(desc.find(L'\n') != desc.npos)
-		    shortdesc = cw::util::transcode(section) + L" - " + wstring(desc, 0, desc.find('\n'));
+		    {
+		      shortdesc = cw::util::transcode(section) + fill + wstring(desc, 0, desc.find('\n'));
+		    }
 		  else
 		    {
-		      shortdesc = cw::util::transcode(section) + desc;
+		      shortdesc = cw::util::transcode(section) + fill + desc;
 		      desc = L"";
 		    }
 		}
@@ -464,7 +459,7 @@ public:
 
 const char * const pkg_grouppolicy_status::state_titles[numgroups] =
 {
-  N_("Security Updates\n Security updates for these packages are available from security.debian.org."),
+  N_("Security Updates\n Security updates for these packages are available from security.debian.org (or mirrors)."),
   N_("Upgradable Packages\n A newer version of these packages is available."),
   N_("New Packages\n These packages have been added to Debian since the last time you cleared the list of \"new\" packages (choose \"Forget new packages\" from the Actions menu to empty this list)."),
   N_("Installed Packages\n These packages are currently installed on your computer."),
@@ -479,21 +474,6 @@ pkg_grouppolicy *pkg_grouppolicy_status_factory::instantiate(pkg_signal *_sig,
   return new pkg_grouppolicy_status(chain, _sig, _desc_sig);
 }
 
-// Stolen from apt-watch:
-
-/** Tests whether a particular version is security-related.
- *
- *  \return \b true iff the given package version comes from security.d.o
- */
-static bool version_is_security(const pkgCache::VerIterator &ver)
-{
-  for(pkgCache::VerFileIterator F=ver.FileList(); !F.end(); ++F)
-    if(string(F.File().Site())=="security.debian.org")
-      return true;
-
-  return false;
-}
-
 void pkg_grouppolicy_status::add_package(const pkgCache::PkgIterator &pkg,
 					 pkg_subtree *root)
 {
@@ -509,7 +489,7 @@ void pkg_grouppolicy_status::add_package(const pkgCache::PkgIterator &pkg,
       else if(state.Status != 2 && state.Upgradable())
 	{
 	  pkgCache::VerIterator candver=state.CandidateVerIter(*apt_cache_file);
-	  if(version_is_security(candver))
+	  if (is_security(candver))
 	    section=security_upgradable;
 	  else
 	    section=upgradable;
